/**
 * COOS - Connected Objects Operating System (www.connectedobjects.org).
 *
 * Copyright (C) 2009 Telenor ASA and Tellu AS. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may also contact one of the following for additional information:
 * Telenor ASA, Snaroyveien 30, N-1331 Fornebu, Norway (www.telenor.no)
 * Tellu AS, Hagalokkveien 13, N-1383 Asker, Norway (www.tellu.no)
 */
package org.coos.plugin.logserver.impl;

import java.util.Hashtable;

import org.coos.actorframe.ActorDomainSM;
import org.coos.actorframe.ActorPartSpec;
import org.coos.actorframe.application.Application;
import org.coos.actorframe.application.DomainSpec;
import org.coos.javaframe.ActorSpec;
import org.coos.javaframe.ApplicationSpec;
import org.coos.javaframe.LoggerFactory;
import org.coos.javaframe.SchedulerSpec;
import org.coos.javaframe.TraceConstants;
import org.coos.messaging.Exchange;
import org.coos.messaging.Message;
import org.coos.messaging.plugin.actorframe.ActorFrameContainer;
import org.coos.messaging.util.Log;
import org.coos.messaging.util.LogFactory;

public class LogServerEndpoint extends ActorFrameContainer {
	private static String LOG_SERVER_DOMAIN_NAME = "LogServerDomain";
	private static String ACTOR_DOMAIN_TYPE = "ActorDomain";
	private static String LOG_SERVER_ACTOR_TYPE = "LogServer";
	private static String LOG_SERVER_ACTOR_ROLE = "logServer";
	static String DEFAULT_CONFIG_FILE = "logback-server.xml";
	static int MAX_ERROR_COUNT = 5;
	static Log LOGGER = LogFactory.getLog(LogServerEndpoint.class);
	private int errorCount = 0;
	private boolean starting = false;
	private boolean started = false;

	@Override
	public void start() {
		starting = true;
		try {
			String configFile = getProperty(LogServerSM.CONFIG_FILE_KEY);
			if (configFile == null) {
				configFile = DEFAULT_CONFIG_FILE;
			}
			setApplication(createApplication(configFile));
			super.start();
			started = true;
			LOGGER.debug("Started endpoint " + getEndpointUuid());
		} catch (Exception e) {
			LOGGER.error("Failed to start LogServer, e = " + e.getMessage());
		} finally {
			starting = false;
		}
	}

	@Override
	public void stop() {
		super.stop();
		started = false;
	}

	@Override
	public void process(Exchange exchange) {
		if (started) {
			Message msg = exchange.getInBoundMessage();
			msg.setReceiverEndpointUri(msg.getReceiverEndpointUri() + "/" + LOG_SERVER_ACTOR_ROLE + "@" + LOG_SERVER_ACTOR_TYPE);
			super.process(exchange);
		} else {
			if (starting) {
				LOGGER.warn("Exchange: " + exchange + " Message ignored as service is starting.");
				return;
			}
			if (errorCount++ < MAX_ERROR_COUNT) {
				LOGGER.error("Exchange: " + exchange + " Message ignored as service failed to start. See above.");
				if (errorCount == MAX_ERROR_COUNT) {
					LOGGER.warn("Repeated errors. Subsequent errors will not be logged.");
				}
			}
		}
	}

	@SuppressWarnings("unchecked")
	private Application createApplication(String configFile) {
		ApplicationSpec appSpec = new ApplicationSpec("LogServerEndpoint");
		ActorSpec actor1 = new ActorSpec(LOG_SERVER_ACTOR_TYPE, LogServerSM.class.getName());
		ActorSpec actor2 = new ActorSpec(ACTOR_DOMAIN_TYPE, ActorDomainSM.class.getName());
		ActorPartSpec aps = new ActorPartSpec(LOG_SERVER_ACTOR_ROLE, LOG_SERVER_ACTOR_TYPE);
		
		// Form properties for actor
		Hashtable h = new Hashtable();
		h.put(LogServerSM.CONFIG_FILE_KEY, configFile);
		aps.getActorProperties().put("default", h);
		
		aps.setTraceLev(TraceConstants.tlDebug);
		actor2.addPartSpec(aps);
		appSpec.addActorSpec(actor1);
		appSpec.addActorSpec(actor2);
		DomainSpec ds = new DomainSpec(LOG_SERVER_DOMAIN_NAME, ACTOR_DOMAIN_TYPE);
		ds.addSchedulerSpec(new SchedulerSpec("default"));
		appSpec.setDomainSpec(ds);
		appSpec.setTraceEnabled(true);
		LoggerFactory.setTrace(true);
		LoggerFactory.setTraceError(true);

		Application app = new Application(appSpec);
		return app;
	}

}
